@@ -9,6 +9,7 @@ from hepi.results import Result
 from hepi.util import DL2DF, LD2DL, DictData, namehash
 from smpl.parallel import par
 import time
+import tqdm
 
 
 class RunParam(DictData):
@@ -75,7 +76,7 @@ class Runner:
     def _prepare(self, p: Input, **kwargs) -> RunParam:
         skip_ = kwargs["skip"]
         d = p.__dict__
-        d["runner"] = str(type(self).__name__) #+ "-" + self.get_version() # TODO re add version, but removed for reusable hashing!
+        d["runner"] = str(type(self).__name__) + "-" + self.get_version() # TODO re add version, but removed for reusable hashing!
         name = namehash("_".join("".join(str(_[0]) + "_" + str(_[1]))
                                  for _ in d.items()).replace("/", "-"))
         #print(name)
@@ -83,9 +84,11 @@ class Runner:
         if skip_ and os.path.isfile(self.get_output_dir() + name +
                                     ".out") and self._is_valid(
                                         self.get_output_dir() + name + ".out",
-                                        p, d):
-            print("skip", end='')
+                                        p, d,**kwargs):
+            print(".", end='')
             skip = True
+        else:
+            print('|', end='')
         return RunParam(execute=self.get_output_dir() + name + ".sh",
                         in_file=self.get_output_dir() + name + ".in",
                         out_file=self.get_output_dir() + name + ".out",
@@ -131,7 +134,7 @@ class Runner:
 		    parallel (bool): Run jobs in parallel.
 		    sleep (int): Sleep seconds after starting job.
             run (bool): Actually start/queue runner.
-            ignore_error (bool): Continue instead of raising Exceptions.
+            ignore_error (bool): Continue instead of raising Exceptions. Also ignores hash collisions.
 
 		Returns:
 		    :obj:`pd.DataFrame` : combined dataframe of results and parameters. The dataframe is empty if `parse` is set to False.
@@ -141,8 +144,8 @@ class Runner:
             warnings.warn("The path is not valid for " + self.get_name())
             if not ignore_error:
                 raise RuntimeError("The path is not valid for " + self.get_name())
-        rps = self._prepare_all(params, parse=parse, skip=skip, **kwargs)
-        print("Running: " + str(len(params)) + " jobs")
+        rps = self._prepare_all(params, parse=parse, skip=skip, ignore_error=ignore_error,**kwargs)
+        print("= " + str(len(params)) + " jobs")
         if sleep is None:
             sleep = 0 if parse else 5
         if run:
@@ -200,7 +203,7 @@ class Runner:
             return output
         return []
 
-    def _is_valid(self, file: str, p: Input, d) -> bool:
+    def _is_valid(self, file: str, p: Input, d, **kwargs) -> bool:
         """
 		Verifies that a file is a complete output.
 	
@@ -235,8 +238,12 @@ class Runner:
 	
 		"""
         rsl = []
-        for r in par(self._parse_file, outputs):
-            rsl.append(r)
+        #for r in par(self._parse_file, outputs):
+        #    rsl.append(r)
+
+        # parallelized opens to many files at times
+        for o in tqdm.tqdm(outputs):
+            rsl.append(self._parse_file(o))
         return rsl
 
     def _parse_file(self, file: str) -> Result:
